@@ -2,7 +2,8 @@ use strict;
 package Inline::Foo;
 
 require Inline;
-@Inline::Foo::ISA = qw(Inline);
+our @ISA = qw(Inline);
+our $VERSION = '0.80_01';
 
 use Carp;
 use File::Spec;
@@ -53,6 +54,7 @@ sub build {
     $code =~ s/bar-//g if $o->{ILSM}{BAR};
     {
         package Foo::Tester;
+        our $VERSION = '0.02';
         eval $code;
     }
     croak "Foo build failed:\n$@" if $@;
@@ -61,6 +63,7 @@ sub build {
     $o->mkpath($path) unless -d $path;
     open FOO_OBJ, "> $obj"
       or croak "Can't open $obj for output\n$!";
+    binmode(FOO_OBJ, ':utf8');
     print FOO_OBJ $code;
     close \*FOO_OBJ;
 }
@@ -70,6 +73,7 @@ sub load {
     my $obj = $o->{API}{location};
     open FOO_OBJ, "< $obj"
       or croak "Can't open $obj for output\n$!";
+    binmode(FOO_OBJ, ':utf8');
     my $code = join '', <FOO_OBJ>;
     close \*FOO_OBJ;
     eval "package $o->{API}{pkg};\n$code";
