@@ -4,17 +4,24 @@ use strict;
 use base 'Exporter';
 use ExtUtils::MakeMaker();
 use Carp;
+use version;
 
 our @EXPORT = qw(WriteMakefile WriteInlineMakefile);
+our $VERSION = '0.80_01';
 
 sub WriteInlineMakefile {
-#    warn <<END;
-#
-#Inline::MakeMaker::WriteInlineMakefile() is deprecated as of Inline-0.44.
-#Inline::MakeMaker::WriteMakefile() should be used instead.
-#
-#END
-    goto &WriteMakefile;
+    carp <<EOF;
+
+======================== DEPRECATION ALERT ======================
+
+WriteInlineMakefile was deprecated in 2002. This warning is from 2014.
+WriteInlineMakefile will soon be removed. Please change this Makefile.PL
+to use WriteMakefile instead.
+
+========================== MESSAGE ENDS =========================
+
+EOF
+     goto &WriteMakefile;
 }
 
 sub WriteMakefile {
@@ -34,10 +41,8 @@ END
         $version = ExtUtils::MM_Unix->parse_version($args{VERSION_FROM})
           or croak "Can't determine version for $name\n";
     }
-    croak <<END unless $version =~ /^\d\.\d\d$/;
-Invalid version '$version' for $name.
-Must be of the form '#.##'. (For instance '1.23')
-END
+    croak "Invalid 'version.pm' version '$version' for '$name'.\n"
+        unless version::is_lax($version);
 
     # Provide a convenience rule to clean up Inline's messes
     $args{clean} = { FILES => "_Inline *.inl " }
@@ -60,6 +65,7 @@ END
                     my ($vol, $dirs, $file) = File::Spec->splitpath(substr($path, length($lib)+1));
                     my @dirs = File::Spec->splitdir($dirs);
                     pop @dirs unless length($dirs[$#dirs]);
+                    next unless ($file =~ /.pm$/);
                     $file =~ s/\.[^.]+$//;
 
                     push @objects, join('::', @dirs, $file);
@@ -87,13 +93,14 @@ END
 
 MAKEFILE
     for (0..$#objects) {
+        # the subdivision in hashes is done to avoid {}, which break dmake
         print MAKEFILE <<MAKEFILE;
-$obj_rules[$_]: \$(TO_INST_PM)
-	\$(PERL) -Mblib -MInline=NOISY,_INSTALL_ -M$objects[$_] -e"Inline::satisfy_makefile_dep({API => {modinlname => '$obj_rules[$_]', module => '$objects[$_]'}});" $version \$(INST_ARCHLIB)
+$obj_rules[$_] : pm_to_blib
+\t\$(PERL) -Mblib -MInline=NOISY,_INSTALL_ -M$objects[$_] -e"my %A = (modinlname => '$obj_rules[$_]', module => '$objects[$_]'); my %S = (API => \\%A); Inline::satisfy_makefile_dep(\\%S);" $version \$(INST_ARCHLIB)
 MAKEFILE
     }
 
-print MAKEFILE "\npure_all :: ",join(' ',@obj_rules),"\n";
+print MAKEFILE "\ndynamic :: ",join(' ',@obj_rules),"\n";
 
 print MAKEFILE <<MAKEFILE;
 
